{% extends "admin/change_list.html" %}
{% block filters %}
<div class="aligned form-row field-taxonomy_term">
        <div style="text-align:center; padding-top: 4px; font-size: 18px; font-style:italic;">
            Tree of {{ cl.opts.verbose_name_plural }}
        </div>
</div>
{% endblock %}


