{% extends "admin/change_form.html" %}
{% block form_top %}
<div class="aligned form-row field-taxonomy_term">
        <div style="text-align:center; padding-top: 4px; font-size: 18px; font-style:italic;">
            Tree of {{ opts.verbose_name_plural }}
        </div>
</div>
{% endblock %}

