#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['taxonomy', 'taxonomy.ttest.migrations']

package_data = \
{'': ['*'],
 'taxonomy': ['templates/*',
              'templates/taxonomy/*',
              'templatetags/*',
              'ttest/*',
              'ttest/tests/*']}

setup(name='django-taxonomy2',
      version='1.0.1',
      description='Category tree for Django. Replicable and customizable. Includes Admin. Low on features, easy setup.',
      author='robert crowther',
      author_email='rw.crowther@gmail.com',
      url='https://github.com/rcrowther/django-taxonomy',
      packages=packages,
      package_data=package_data,
     )
